@@ -45,8 +45,17 @@ def exc(request):
 def notfound(request):
     raise HTTPNotFound()
 
+@view_config(route_name='test_ajax', renderer='__main__:templates/ajax.mako')
+def test_ajax(request):
+    return {}
+
+@view_config(route_name='call_ajax', renderer='json')
+def call_ajax(request):
+    return {'ajax':'success'}
+
 @view_config(context=HTTPNotFound, renderer='__main__:templates/notfound.mako')
 def notfound_view(request):
+    request.response.status_code = 404
     return {}
 
 @view_config(renderer='__main__:templates/index.mako') # found via traversal
@@ -68,18 +77,18 @@ def test_highorder(request):
     return {}
 
 @view_config(route_name='test_predicates',
-        renderer='__main__:templates/index.mako')
+             renderer='__main__:templates/index.mako')
 def test_predicates(request):
-    return {'title':'Test route predicates'}
+    return {'title': 'Test route predicates'}
 
-@view_config(route_name='test_mako_exc',
-        renderer='__main__:templates/error.mako')
 @view_config(route_name='test_chameleon_exc',
-        renderer='__main__:templates/error.pt')
+             renderer='__main__:templates/error.pt')
+@view_config(route_name='test_mako_exc',
+             renderer='__main__:templates/error.mako')
 @view_config(route_name='test_jinja2_exc',
-        renderer='__main__:templates/error.jinja2')
+             renderer='__main__:templates/error.jinja2')
 def test_template_exc(request):
-    return {'title':'Test template exceptions'}
+    return {'title': 'Test template exceptions'}
 
 class DummyRootFactory(object):
     def __init__(self, request):
@@ -117,10 +126,13 @@ if __name__ == '__main__':
     config.add_route('test_mako_exc', '/mako_exc')
     config.add_route('test_jinja2_exc', '/jinja2_exc')
     config.add_route('test_highorder', text_(b'/La Pe\xc3\xb1a', 'utf-8'))
+    config.add_route('test_ajax', '/ajax')
+    config.add_route('call_ajax', '/call_ajax')
     config.scan('__main__')
     config.include('pyramid_chameleon')
     config.include('pyramid_jinja2')
     config.include('pyramid_mako')
+
     if sqlalchemy:
         config.include('sqla')
     config.include('pyramid_debugtoolbar')
